@@ -10,31 +10,40 @@ This can be used for creation of Python REPLs.
 from __future__ import unicode_literals
 
 from prompt_toolkit import AbortAction
+from prompt_toolkit.auto_suggest import AutoSuggestFromHistory, ConditionalAutoSuggest
 from prompt_toolkit.buffer import Buffer
 from prompt_toolkit.document import Document
-from prompt_toolkit.enums import DEFAULT_BUFFER
+from prompt_toolkit.enums import DEFAULT_BUFFER, EditingMode
 from prompt_toolkit.filters import Condition, Always
 from prompt_toolkit.history import FileHistory, InMemoryHistory
 from prompt_toolkit.interface import CommandLineInterface, Application, AcceptAction
-from prompt_toolkit.key_binding.manager import KeyBindingManager
+from prompt_toolkit.key_binding.defaults import load_key_bindings_for_prompt, load_mouse_bindings
+from prompt_toolkit.key_binding.vi_state import InputMode
+from prompt_toolkit.key_binding.registry import MergedRegistry, ConditionalRegistry
 from prompt_toolkit.layout.lexers import PygmentsLexer
-from prompt_toolkit.utils import Callback, is_windows
-from prompt_toolkit.validation import SwitchableValidator
+from prompt_toolkit.shortcuts import create_output
+from prompt_toolkit.styles import DynamicStyle
+from prompt_toolkit.utils import is_windows
+from prompt_toolkit.validation import ConditionalValidator
 
-from ptpython.completer import PythonCompleter
-from ptpython.key_bindings import load_python_bindings, load_sidebar_bindings, load_confirm_exit_bindings
-from ptpython.layout import create_layout, CompletionVisualisation
-from ptpython.style import get_all_code_styles, get_all_ui_styles, generate_style
-from ptpython.utils import get_jedi_script_from_document, document_is_multiline_python
-from ptpython.validator import PythonValidator
-from ptpython.prompt_style import IPythonPrompt, ClassicPrompt
+from .completer import PythonCompleter
+from .history_browser import create_history_application
+from .key_bindings import load_python_bindings, load_sidebar_bindings, load_confirm_exit_bindings
+from .layout import create_layout, CompletionVisualisation
+from .prompt_style import IPythonPrompt, ClassicPrompt
+from .style import get_all_code_styles, get_all_ui_styles, generate_style
+from .utils import get_jedi_script_from_document, document_is_multiline_python
+from .validator import PythonValidator
 
 from functools import partial
-from pygments.lexers import PythonLexer
 
 import six
 import __future__
 
+if six.PY2:
+    from pygments.lexers import PythonLexer
+else:
+    from pygments.lexers import Python3Lexer as PythonLexer
 
 __all__ = (
     'PythonInput',
@@ -115,7 +124,7 @@ class PythonInput(object):
 
         python_input = PythonInput(...)
         application = python_input.create_application()
-        cli = CommandLineInterface(application=application)
+        cli = PythonCommandLineInterface(application=application)
         python_code = cli.run()
     """
     def __init__(self,
@@ -149,14 +158,15 @@ class PythonInput(object):
         self._extra_buffer_processors = _extra_buffer_processors or []
 
         # Settings.
-        self.show_signature = True
+        self.show_signature = False
         self.show_docstring = False
         self.show_meta_enter_message = True
         self.completion_visualisation = CompletionVisualisation.MULTI_COLUMN
         self.completion_menu_scroll_offset = 1
 
-        self.show_line_numbers = True
+        self.show_line_numbers = False
         self.show_status_bar = True
+        self.wrap_lines = True
         self.complete_while_typing = True
         self.vi_mode = vi_mode
         self.paste_mode = False  # When True, don't insert whitespace after newline.
@@ -166,20 +176,23 @@ class PythonInput(object):
         self.enable_open_in_editor = True
         self.enable_system_bindings = True
         self.enable_input_validation = True
+        self.enable_auto_suggest = False
+        self.enable_mouse_support = False
         self.enable_history_search = False  # When True, like readline, going
                                             # back in history will filter the
                                             # history on the records starting
                                             # with the current input.
 
-        self.highlight_matching_parenthesis = True
+        self.highlight_matching_parenthesis = False
         self.show_sidebar = False  # Currently show the sidebar.
         self.show_sidebar_help = True # When the sidebar is visible, also show the help text.
         self.show_exit_confirmation = False  # Currently show 'Do you really want to exit?'
         self.terminal_title = None  # The title to be displayed in the terminal. (None or string.)
         self.exit_message = 'Do you really want to exit?'
+        self.insert_blank_line_after_output = True  # (For the REPL.)
 
         # Tokens to be shown at the prompt.
-        self.prompt_style = 'ipython'  # The currently active style.
+        self.prompt_style = 'classic'  # The currently active style.
 
         self.all_prompt_styles = {  # Styles selectable from the menu.
             'ipython': IPythonPrompt(self),
@@ -202,6 +215,7 @@ class PythonInput(object):
             self._current_code_style_name = 'win32'
 
         self._current_style = self._generate_style()
+        self.true_color = False
 
         # Options to be configurable from the sidebar.
         self.options = self._create_options()
@@ -213,19 +227,25 @@ class PythonInput(object):
         # Code signatures. (This is set asynchronously after a timeout.)
         self.signatures = []
 
-        # Use a KeyBindingManager for loading the key bindings.
-        self.key_bindings_manager = KeyBindingManager(
-            enable_vi_mode=Condition(lambda cli: self.vi_mode),
-            enable_open_in_editor=Condition(lambda cli: self.enable_open_in_editor),
-            enable_system_bindings=Condition(lambda cli: self.enable_system_bindings),
+        # Create a Registry for the key bindings.
+        self.key_bindings_registry = MergedRegistry([
+            ConditionalRegistry(
+                registry=load_key_bindings_for_prompt(
+                    enable_abort_and_exit_bindings=True,
+                    enable_search=True,
+                    enable_open_in_editor=Condition(lambda cli: self.enable_open_in_editor),
+                    enable_system_bindings=Condition(lambda cli: self.enable_system_bindings),
+                    enable_auto_suggest_bindings=Condition(lambda cli: self.enable_auto_suggest)),
 
-            # Disable all default key bindings when the sidebar or the exit confirmation
-            # are shown.
-            enable_all=Condition(lambda cli: not (self.show_sidebar or self.show_exit_confirmation)))
-
-        load_python_bindings(self.key_bindings_manager, self)
-        load_sidebar_bindings(self.key_bindings_manager, self)
-        load_confirm_exit_bindings(self.key_bindings_manager, self)
+                # Disable all default key bindings when the sidebar or the exit confirmation
+                # are shown.
+                filter=Condition(lambda cli: not (self.show_sidebar or self.show_exit_confirmation))
+            ),
+            load_mouse_bindings(),
+            load_python_bindings(self),
+            load_sidebar_bindings(self),
+            load_confirm_exit_bindings(self),
+        ])
 
         # Boolean indicating whether we have a signatures thread running.
         # (Never run more than one at the same time.)
@@ -260,10 +280,6 @@ class PythonInput(object):
 
         return flags
 
-    @property
-    def key_bindings_registry(self):
-        return self.key_bindings_manager.registry
-
     @property
     def add_key_binding(self):
         """
@@ -389,6 +405,10 @@ class PythonInput(object):
                            'on': lambda: enable('enable_history_search') and disable('complete_while_typing'),
                            'off': lambda: disable('enable_history_search'),
                        }),
+                simple_option(title='Mouse support',
+                              description='Respond to mouse clicks and scrolling for positioning the cursor, '
+                                          'selecting text and scrolling through windows.',
+                              field_name='enable_mouse_support'),
                 simple_option(title='Confirm on exit',
                               description='Require confirmation when exiting.',
                               field_name='confirm_exit'),
@@ -396,6 +416,10 @@ class PythonInput(object):
                               description='In case of syntax errors, move the cursor to the error '
                                           'instead of showing a traceback of a SyntaxError.',
                               field_name='enable_input_validation'),
+                simple_option(title='Auto suggestion',
+                              description='Auto suggest inputs by looking at the history. '
+                                          'Pressing right arrow or Ctrl-E will complete the entry.',
+                              field_name='enable_auto_suggest'),
                 Option(title='Accept input on enter',
                        description='Amount of ENTER presses required to execute input when the cursor '
                                    'is at the end of the input. (Note that META+ENTER will always execute.)',
@@ -418,9 +442,12 @@ class PythonInput(object):
                            CompletionVisualisation.TOOLBAR: lambda: enable('completion_visualisation', CompletionVisualisation.TOOLBAR),
                        }),
                 Option(title='Prompt',
-                       description='Visualisation of the prompt.',
+                       description="Visualisation of the prompt. ('>>>' or 'In [1]:')",
                        get_current_value=lambda: self.prompt_style,
                        get_values=lambda: dict((s, partial(enable, 'prompt_style', s)) for s in self.all_prompt_styles)),
+                simple_option(title='Blank line after output',
+                              description='Insert a blank line after the output.',
+                              field_name='insert_blank_line_after_output'),
                 simple_option(title='Show signature',
                               description='Display function signatures.',
                               field_name='show_signature'),
@@ -434,6 +461,9 @@ class PythonInput(object):
                               description='Show the [Meta+Enter] message when this key combination is required to execute commands. ' +
                                   '(This is the case when a simple [Enter] key press will insert a newline.',
                               field_name='show_meta_enter_message'),
+                simple_option(title='Wrap lines',
+                              description='Wrap lines instead of scrolling horizontally.',
+                              field_name='wrap_lines'),
                 simple_option(title='Show status bar',
                               description='Show the status bar at the bottom of the terminal.',
                               field_name='show_status_bar'),
@@ -457,6 +487,9 @@ class PythonInput(object):
                        get_values=lambda: dict(
                             (name, partial(self.use_ui_colorscheme, name)) for name in self.ui_styles)
                        ),
+                simple_option(title='True color (24 bit)',
+                              description='Use 24 bit colors instead of 265 colors',
+                              field_name='true_color'),
             ]),
         ]
 
@@ -472,7 +505,6 @@ class PythonInput(object):
         return Application(
             layout=create_layout(
                 self,
-                self.key_bindings_manager,
                 lexer=self._lexer,
                 input_buffer_height=self._input_buffer_height,
                 extra_buffer_processors=self._extra_buffer_processors,
@@ -482,12 +514,15 @@ class PythonInput(object):
             buffers=buffers,
             key_bindings_registry=self.key_bindings_registry,
             paste_mode=Condition(lambda cli: self.paste_mode),
+            mouse_support=Condition(lambda cli: self.enable_mouse_support),
             on_abort=AbortAction.RETRY,
             on_exit=self._on_exit,
-            get_style=lambda: self._current_style,
+            style=DynamicStyle(lambda: self._current_style),
             get_title=lambda: self.terminal_title,
+            reverse_vi_search_direction=True,
+            on_initialize=self._on_cli_initialize,
             on_start=self._on_start,
-            on_input_timeout=Callback(self._on_input_timeout))
+            on_input_timeout=self._on_input_timeout)
 
     def _create_buffer(self):
         """
@@ -505,19 +540,37 @@ class PythonInput(object):
             tempfile_suffix='.py',
             history=self.history,
             completer=self._completer,
-            validator=SwitchableValidator(
+            validator=ConditionalValidator(
                 self._validator,
                 Condition(lambda: self.enable_input_validation)),
+            auto_suggest=ConditionalAutoSuggest(
+                AutoSuggestFromHistory(),
+                Condition(lambda cli: self.enable_auto_suggest)),
             accept_action=self._accept_action)
 
         return python_buffer
 
+    def _on_cli_initialize(self, cli):
+        """
+        Called when a CommandLineInterface has been created.
+        """
+        # Synchronize PythonInput state with the CommandLineInterface.
+        def synchronize(_=None):
+            if self.vi_mode:
+                cli.editing_mode = EditingMode.VI
+            else:
+                cli.editing_mode = EditingMode.EMACS
+
+        cli.input_processor.beforeKeyPress += synchronize
+        cli.input_processor.afterKeyPress += synchronize
+        synchronize()
+
     def _on_input_timeout(self, cli):
         """
         When there is no input activity,
         in another thread, get the signature of the current code.
         """
-        if cli.focus_stack.current != DEFAULT_BUFFER:
+        if cli.current_buffer_name != DEFAULT_BUFFER:
             return
 
         # Never run multiple get-signature threads.
@@ -582,13 +635,34 @@ class PythonInput(object):
         cli.eventloop.run_in_executor(run)
 
     def on_reset(self, cli):
-        self.key_bindings_manager.reset()
         self.signatures = []
 
+    def enter_history(self, cli):
+        """
+        Display the history.
+        """
+        cli.vi_state.input_mode = InputMode.NAVIGATION
+
+        def done(result):
+            if result is not None:
+                cli.buffers[DEFAULT_BUFFER].document = result
+
+            cli.vi_state.input_mode = InputMode.INSERT
+
+        cli.run_sub_application(create_history_application(
+            self, cli.buffers[DEFAULT_BUFFER].document), done)
+
 
 class PythonCommandLineInterface(CommandLineInterface):
-    def __init__(self, eventloop=None, input=None, output=None):
-        python_input = PythonInput()
+    def __init__(self, eventloop=None, python_input=None, input=None, output=None):
+        assert python_input is None or isinstance(python_input, PythonInput)
+
+        python_input = python_input or PythonInput()
+
+        # Make sure that the prompt_toolkit 'renderer' knows about the
+        # 'true_color' property of PythonInput.
+        if output is None:
+            output=create_output(true_color=Condition(lambda: python_input.true_color))
 
         super(PythonCommandLineInterface, self).__init__(
             application=python_input.create_application(),
